import re
import os
import matplotlib.pyplot as plt
import numpy as np
from datetime import datetime, timedelta
from cycler import cycler


directory_path = r"C:\Users\Braan\OneDrive - NTNU\10_V2024\Project\ProjectData"#"C:\Users\Braan\Desktop\ProjectData"

time_unit = 'h' #s: second; m: minute; h: hour
time_read_start = 10

#Export inputs an index in the directory_path, and plots the data.
def export_data(file_index):    
    
    # Finds the files in the directory_path if the path exists
    def check_directory(directory_path):
        if os.path.exists(directory_path):
            files = os.listdir(directory_path)
            if __name__=='__main__':
                print(f"Directory {directory_path} exists.")
                print(f"Files in the directory: {files}")
            return files
        else:
            print(f"Directory {directory_path} does not exist.")
            return None
    
    # Reads the raw file. Not for plotting
    def read_file(file_path):
        with open(file_path, 'r') as file:
            return file.read()
    

    
    # The timestamps in the raw file takes the live hour when the simulation is completed. This function sets the initial time to 0, and converts from 
    # '%m-%d-%Y %H:%M:%S using the "datetime" package to time in one unit (s, m, h). i.e. a simplification.
    def convert_timestamps(timestamps, unit='s'):
        conversion_factors = {'s': 1, 'm': 60, 'h': 3600}
        try:
            conversion_factor = conversion_factors[unit.lower()]
        except KeyError:
            raise ValueError("Invalid unit. Use 's', 'm', or 'h'.")
    
        converted_timestamps = []
        prev_date = None
    
        for timestamp in timestamps:
            time_obj = datetime.strptime(timestamp, '%m-%d-%Y %H:%M:%S')
            #Time conversion assuming a constant relationship between [yr, month, day, min] to second
            converted_value = time_obj.year*31536000 + time_obj.month*31536000/12 + time_obj.day*3600*24 + time_obj.hour * 3600 + time_obj.minute * 60 + time_obj.second
            converted_timestamps.append(converted_value / conversion_factor)
    
            prev_date = time_obj.date()
    
        return converted_timestamps
    
    # Extracts the aspen stream name and variable. Example: STREAMS("D").Zn("METHANOL")
    def extract_variable_names(file_content):
        pattern = re.compile(r'===\n(.*?)===', re.DOTALL)
        matches = pattern.findall(file_content)
        return re.findall(r'~~~(.*?)~~~', matches[-1]) if matches else []
    
    # the general pattern for each line used to generate data and time
    def generate_pattern(n):
        base_pattern = r'(\d{2}-\d{2}-\d{4} \d{2}:\d{2}:\d{2})'
        variable_pattern = r',([0-9.]+),G'
        return base_pattern + (variable_pattern * n)
    
    
    files = check_directory(directory_path)
    if files:
        filex = files[file_index]
        data = read_file(os.path.join(directory_path, filex))
    
        variable_names = extract_variable_names(data)
        nums = len(variable_names)
        matches = re.findall(generate_pattern(nums), data)
    
        data_tuples = [(time, *(float(num) for num in nums)) for time, *nums in matches]
    
        timestamps = [time for time, *_ in matches] #time raw data
        variables = [[float(num) for num in nums] for _, *nums in matches] 
        
        # Find the index of the number closest to the input value
        closest_time_index = np.abs(np.array(convert_timestamps(timestamps, time_unit)) - time_read_start).argmin()


        t0 = convert_timestamps(timestamps, time_unit)[0] #first timestamp
        t = [x-t0 for x in convert_timestamps(timestamps, time_unit)] #sets initial time to zero.
        
        #Test if the plot script works!
        if __name__ == "__main__":
            # Plot each variable against time in minutes
            plt.figure(figsize=(10, 5))
            for var_name, var_list in zip(variable_names, zip(*variables)):
                print(var_name)
                
                plt.plot(t, var_list, linewidth=2, label=var_name)
                
            plt.xlabel(f'Time ({time_unit})')
            plt.ylabel('')
            plt.xticks(rotation=0)
            plt.tight_layout()
        
            line_cycler   = (cycler(color=["red", "#56B4E9", "#009E73", "#0072B2", "#D55E00", "#CC79A7", "#F0E442"]) +
                             cycler(linestyle=["-", "-", "-.", ":", "-", "--", "-."]))
            marker_cycler = (cycler(color=["#E69F00", "#56B4E9", "#009E73", "#0072B2", "#D55E00", "#CC79A7", "#F0E442"]) +
                             cycler(linestyle=["none", "none", "none", "none", "none", "none", "none"]) +
                             cycler(marker=["4", "2", "3", "1", "+", "x", "."]))
        
            plt.rc("axes", prop_cycle=line_cycler)
        
            plt.grid(which='major', linestyle='-', linewidth='0.6', color='black', alpha=0.3)
            plt.grid(which='minor', linestyle=':', linewidth='0.6', color='black', alpha=0.3)
            plt.minorticks_on()
        
            plt.legend(loc='best')
            plt.show()
    
    data = {'x':t,
            'y':variables,
            'y_names': variable_names
            }
    return data

file_index = 5
export_data(file_index)